namespace Submarine {
	
	public struct LanguageInfo {
		public string long_code { get; private set; }
		public string? long_code_alt { get; private set; } //if this is set, then long_code has bibliographical meaning and long_code_alt terminological meaning
		public string? short_code { get; private set; }
		public string name { get; private set; }
		
		
		public LanguageInfo(string long_code, string long_code_alt, string short_code, string name) {
			this.long_code = long_code;
			this.long_code_alt = long_code_alt;
			this.short_code = short_code;
			this.name = name;
		}
	}
	
	private Gee.List<string> all_language_codes = null;
	private Gee.Map<string, LanguageInfo?> language_infos = null;
	
	public Gee.List<string> get_language_codes() {
		if(all_language_codes == null) {
			all_language_codes = new Gee.ArrayList<string>();
			for(int i=0; i<LANGUAGE_INFOS_ARRAY.length; i++) {
				var language_info = LANGUAGE_INFOS_ARRAY[i];
				all_language_codes.add(language_info.long_code);
				if(language_info.long_code_alt != null) {
					all_language_codes.add(language_info.long_code_alt);
				}
				if(language_info.short_code != null) {
					all_language_codes.add(language_info.short_code);
				}
			}
			all_language_codes.sort( (a, b) => {return ((string)a).ascii_casecmp((string)b);} );
		}
		
		return all_language_codes.read_only_view;
	}
	
	public LanguageInfo? get_language_info(string language_code) {
		if(language_infos == null) {
			language_infos = new Gee.HashMap<string, LanguageInfo?>();
			for(int i=0; i<LANGUAGE_INFOS_ARRAY.length; i++) {
				var language_info = LANGUAGE_INFOS_ARRAY[i];
				language_infos.set(language_info.long_code, language_info);
				if(language_info.long_code_alt != null) {
					language_infos.set(language_info.long_code_alt, language_info);
				}
				if(language_info.short_code != null) {
					language_infos.set(language_info.short_code, language_info);
				}
			}
		}
		
		if(language_code in language_infos.keys) {
			return language_infos[language_code];
		}
		
		return null;
	}
	
	//parsed from http://www.loc.gov/standards/iso639-2/php/code_list.php
	private const LanguageInfo[] LANGUAGE_INFOS_ARRAY = {
		{"aar",	null,	"aa",	"Afar"},
		{"abk",	null,	"ab",	"Abkhazian"},
		{"ace",	null,	null,	"Achinese"},
		{"ach",	null,	null,	"Acoli"},
		{"ada",	null,	null,	"Adangme"},
		{"ady",	null,	null,	"Adyghe; Adygei"},
		{"afa",	null,	null,	"Afro-Asiatic languages"},
		{"afh",	null,	null,	"Afrihili"},
		{"afr",	null,	"af",	"Afrikaans"},
		{"ain",	null,	null,	"Ainu"},
		{"aka",	null,	"ak",	"Akan"},
		{"akk",	null,	null,	"Akkadian"},
		{"alb",	"sqi",	"sq",	"Albanian"},
		{"ale",	null,	null,	"Aleut"},
		{"alg",	null,	null,	"Algonquian languages"},
		{"alt",	null,	null,	"Southern Altai"},
		{"amh",	null,	"am",	"Amharic"},
		{"ang",	null,	null,	"English, Old (ca.450-1100)"},
		{"anp",	null,	null,	"Angika"},
		{"apa",	null,	null,	"Apache languages"},
		{"ara",	null,	"ar",	"Arabic"},
		{"arc",	null,	null,	"Official Aramaic (700-300 BCE); Imperial Aramaic (700-300 BCE)"},
		{"arg",	null,	"an",	"Aragonese"},
		{"arm",	"hye",	"hy",	"Armenian"},
		{"arn",	null,	null,	"Mapudungun; Mapuche"},
		{"arp",	null,	null,	"Arapaho"},
		{"art",	null,	null,	"Artificial languages"},
		{"arw",	null,	null,	"Arawak"},
		{"asm",	null,	"as",	"Assamese"},
		{"ast",	null,	null,	"Asturian; Bable; Leonese; Asturleonese"},
		{"ath",	null,	null,	"Athapascan languages"},
		{"aus",	null,	null,	"Australian languages"},
		{"ava",	null,	"av",	"Avaric"},
		{"ave",	null,	"ae",	"Avestan"},
		{"awa",	null,	null,	"Awadhi"},
		{"aym",	null,	"ay",	"Aymara"},
		{"aze",	null,	"az",	"Azerbaijani"},
		{"bad",	null,	null,	"Banda languages"},
		{"bai",	null,	null,	"Bamileke languages"},
		{"bak",	null,	"ba",	"Bashkir"},
		{"bal",	null,	null,	"Baluchi"},
		{"bam",	null,	"bm",	"Bambara"},
		{"ban",	null,	null,	"Balinese"},
		{"baq",	"eus",	"eu",	"Basque"},
		{"bas",	null,	null,	"Basa"},
		{"bat",	null,	null,	"Baltic languages"},
		{"bej",	null,	null,	"Beja; Bedawiyet"},
		{"bel",	null,	"be",	"Belarusian"},
		{"bem",	null,	null,	"Bemba"},
		{"ben",	null,	"bn",	"Bengali"},
		{"ber",	null,	null,	"Berber languages"},
		{"bho",	null,	null,	"Bhojpuri"},
		{"bih",	null,	"bh",	"Bihari languages"},
		{"bik",	null,	null,	"Bikol"},
		{"bin",	null,	null,	"Bini; Edo"},
		{"bis",	null,	"bi",	"Bislama"},
		{"bla",	null,	null,	"Siksika"},
		{"bnt",	null,	null,	"Bantu languages"},
		{"tib",	"bod",	"bo",	"Tibetan"},
		{"bos",	null,	"bs",	"Bosnian"},
		{"bra",	null,	null,	"Braj"},
		{"bre",	null,	"br",	"Breton"},
		{"btk",	null,	null,	"Batak languages"},
		{"bua",	null,	null,	"Buriat"},
		{"bug",	null,	null,	"Buginese"},
		{"bul",	null,	"bg",	"Bulgarian"},
		{"bur",	"mya",	"my",	"Burmese"},
		{"byn",	null,	null,	"Blin; Bilin"},
		{"cad",	null,	null,	"Caddo"},
		{"cai",	null,	null,	"Central American Indian languages"},
		{"car",	null,	null,	"Galibi Carib"},
		{"cat",	null,	"ca",	"Catalan; Valencian"},
		{"cau",	null,	null,	"Caucasian languages"},
		{"ceb",	null,	null,	"Cebuano"},
		{"cel",	null,	null,	"Celtic languages"},
		{"cze",	"ces",	"cs",	"Czech"},
		{"cha",	null,	"ch",	"Chamorro"},
		{"chb",	null,	null,	"Chibcha"},
		{"che",	null,	"ce",	"Chechen"},
		{"chg",	null,	null,	"Chagatai"},
		{"chi",	"zho",	"zh",	"Chinese"},
		{"chk",	null,	null,	"Chuukese"},
		{"chm",	null,	null,	"Mari"},
		{"chn",	null,	null,	"Chinook jargon"},
		{"cho",	null,	null,	"Choctaw"},
		{"chp",	null,	null,	"Chipewyan; Dene Suline"},
		{"chr",	null,	null,	"Cherokee"},
		{"chu",	null,	"cu",	"Church Slavic; Old Slavonic; Church Slavonic; Old Bulgarian; Old Church Slavonic"},
		{"chv",	null,	"cv",	"Chuvash"},
		{"chy",	null,	null,	"Cheyenne"},
		{"cmc",	null,	null,	"Chamic languages"},
		{"cop",	null,	null,	"Coptic"},
		{"cor",	null,	"kw",	"Cornish"},
		{"cos",	null,	"co",	"Corsican"},
		{"cpe",	null,	null,	"Creoles and pidgins, English based"},
		{"cpf",	null,	null,	"Creoles and pidgins, French-based"},
		{"cpp",	null,	null,	"Creoles and pidgins, Portuguese-based"},
		{"cre",	null,	"cr",	"Cree"},
		{"crh",	null,	null,	"Crimean Tatar; Crimean Turkish"},
		{"crp",	null,	null,	"Creoles and pidgins"},
		{"csb",	null,	null,	"Kashubian"},
		{"cus",	null,	null,	"Cushitic languages"},
		{"wel",	"cym",	"cy",	"Welsh"},
		{"dak",	null,	null,	"Dakota"},
		{"dan",	null,	"da",	"Danish"},
		{"dar",	null,	null,	"Dargwa"},
		{"day",	null,	null,	"Land Dayak languages"},
		{"del",	null,	null,	"Delaware"},
		{"den",	null,	null,	"Slave (Athapascan)"},
		{"ger",	"deu",	"de",	"German"},
		{"dgr",	null,	null,	"Dogrib"},
		{"din",	null,	null,	"Dinka"},
		{"div",	null,	"dv",	"Divehi; Dhivehi; Maldivian"},
		{"doi",	null,	null,	"Dogri"},
		{"dra",	null,	null,	"Dravidian languages"},
		{"dsb",	null,	null,	"Lower Sorbian"},
		{"dua",	null,	null,	"Duala"},
		{"dum",	null,	null,	"Dutch, Middle (ca.1050-1350)"},
		{"dut",	"nld",	"nl",	"Dutch; Flemish"},
		{"dyu",	null,	null,	"Dyula"},
		{"dzo",	null,	"dz",	"Dzongkha"},
		{"efi",	null,	null,	"Efik"},
		{"egy",	null,	null,	"Egyptian (Ancient)"},
		{"eka",	null,	null,	"Ekajuk"},
		{"gre",	"ell",	"el",	"Greek, Modern (1453-)"},
		{"elx",	null,	null,	"Elamite"},
		{"eng",	null,	"en",	"English"},
		{"enm",	null,	null,	"English, Middle (1100-1500)"},
		{"epo",	null,	"eo",	"Esperanto"},
		{"est",	null,	"et",	"Estonian"},
		{"ewe",	null,	"ee",	"Ewe"},
		{"ewo",	null,	null,	"Ewondo"},
		{"fan",	null,	null,	"Fang"},
		{"fao",	null,	"fo",	"Faroese"},
		{"per",	"fas",	"fa",	"Persian"},
		{"fat",	null,	null,	"Fanti"},
		{"fij",	null,	"fj",	"Fijian"},
		{"fil",	null,	null,	"Filipino; Pilipino"},
		{"fin",	null,	"fi",	"Finnish"},
		{"fiu",	null,	null,	"Finno-Ugrian languages"},
		{"fon",	null,	null,	"Fon"},
		{"fre",	"fra",	"fr",	"French"},
		{"frm",	null,	null,	"French, Middle (ca.1400-1600)"},
		{"fro",	null,	null,	"French, Old (842-ca.1400)"},
		{"frr",	null,	null,	"Northern Frisian"},
		{"frs",	null,	null,	"Eastern Frisian"},
		{"fry",	null,	"fy",	"Western Frisian"},
		{"ful",	null,	"ff",	"Fulah"},
		{"fur",	null,	null,	"Friulian"},
		{"gaa",	null,	null,	"Ga"},
		{"gay",	null,	null,	"Gayo"},
		{"gba",	null,	null,	"Gbaya"},
		{"gem",	null,	null,	"Germanic languages"},
		{"geo",	"kat",	"ka",	"Georgian"},
		{"gez",	null,	null,	"Geez"},
		{"gil",	null,	null,	"Gilbertese"},
		{"gla",	null,	"gd",	"Gaelic; Scottish Gaelic"},
		{"gle",	null,	"ga",	"Irish"},
		{"glg",	null,	"gl",	"Galician"},
		{"glv",	null,	"gv",	"Manx"},
		{"gmh",	null,	null,	"German, Middle High (ca.1050-1500)"},
		{"goh",	null,	null,	"German, Old High (ca.750-1050)"},
		{"gon",	null,	null,	"Gondi"},
		{"gor",	null,	null,	"Gorontalo"},
		{"got",	null,	null,	"Gothic"},
		{"grb",	null,	null,	"Grebo"},
		{"grc",	null,	null,	"Greek, Ancient (to 1453)"},
		{"grn",	null,	"gn",	"Guarani"},
		{"gsw",	null,	null,	"Swiss German; Alemannic; Alsatian"},
		{"guj",	null,	"gu",	"Gujarati"},
		{"gwi",	null,	null,	"Gwich'in"},
		{"hai",	null,	null,	"Haida"},
		{"hat",	null,	"ht",	"Haitian; Haitian Creole"},
		{"hau",	null,	"ha",	"Hausa"},
		{"haw",	null,	null,	"Hawaiian"},
		{"heb",	null,	"he",	"Hebrew"},
		{"her",	null,	"hz",	"Herero"},
		{"hil",	null,	null,	"Hiligaynon"},
		{"him",	null,	null,	"Himachali languages; Western Pahari languages"},
		{"hin",	null,	"hi",	"Hindi"},
		{"hit",	null,	null,	"Hittite"},
		{"hmn",	null,	null,	"Hmong; Mong"},
		{"hmo",	null,	"ho",	"Hiri Motu"},
		{"hrv",	null,	"hr",	"Croatian"},
		{"hsb",	null,	null,	"Upper Sorbian"},
		{"hun",	null,	"hu",	"Hungarian"},
		{"hup",	null,	null,	"Hupa"},
		{"iba",	null,	null,	"Iban"},
		{"ibo",	null,	"ig",	"Igbo"},
		{"ice",	"isl",	"is",	"Icelandic"},
		{"ido",	null,	"io",	"Ido"},
		{"iii",	null,	"ii",	"Sichuan Yi; Nuosu"},
		{"ijo",	null,	null,	"Ijo languages"},
		{"iku",	null,	"iu",	"Inuktitut"},
		{"ile",	null,	"ie",	"Interlingue; Occidental"},
		{"ilo",	null,	null,	"Iloko"},
		{"ina",	null,	"ia",	"Interlingua (International Auxiliary Language Association)"},
		{"inc",	null,	null,	"Indic languages"},
		{"ind",	null,	"id",	"Indonesian"},
		{"ine",	null,	null,	"Indo-European languages"},
		{"inh",	null,	null,	"Ingush"},
		{"ipk",	null,	"ik",	"Inupiaq"},
		{"ira",	null,	null,	"Iranian languages"},
		{"iro",	null,	null,	"Iroquoian languages"},
		{"ita",	null,	"it",	"Italian"},
		{"jav",	null,	"jv",	"Javanese"},
		{"jbo",	null,	null,	"Lojban"},
		{"jpn",	null,	"ja",	"Japanese"},
		{"jpr",	null,	null,	"Judeo-Persian"},
		{"jrb",	null,	null,	"Judeo-Arabic"},
		{"kaa",	null,	null,	"Kara-Kalpak"},
		{"kab",	null,	null,	"Kabyle"},
		{"kac",	null,	null,	"Kachin; Jingpho"},
		{"kal",	null,	"kl",	"Kalaallisut; Greenlandic"},
		{"kam",	null,	null,	"Kamba"},
		{"kan",	null,	"kn",	"Kannada"},
		{"kar",	null,	null,	"Karen languages"},
		{"kas",	null,	"ks",	"Kashmiri"},
		{"kau",	null,	"kr",	"Kanuri"},
		{"kaw",	null,	null,	"Kawi"},
		{"kaz",	null,	"kk",	"Kazakh"},
		{"kbd",	null,	null,	"Kabardian"},
		{"kha",	null,	null,	"Khasi"},
		{"khi",	null,	null,	"Khoisan languages"},
		{"khm",	null,	"km",	"Central Khmer"},
		{"kho",	null,	null,	"Khotanese; Sakan"},
		{"kik",	null,	"ki",	"Kikuyu; Gikuyu"},
		{"kin",	null,	"rw",	"Kinyarwanda"},
		{"kir",	null,	"ky",	"Kirghiz; Kyrgyz"},
		{"kmb",	null,	null,	"Kimbundu"},
		{"kok",	null,	null,	"Konkani"},
		{"kom",	null,	"kv",	"Komi"},
		{"kon",	null,	"kg",	"Kongo"},
		{"kor",	null,	"ko",	"Korean"},
		{"kos",	null,	null,	"Kosraean"},
		{"kpe",	null,	null,	"Kpelle"},
		{"krc",	null,	null,	"Karachay-Balkar"},
		{"krl",	null,	null,	"Karelian"},
		{"kro",	null,	null,	"Kru languages"},
		{"kru",	null,	null,	"Kurukh"},
		{"kua",	null,	"kj",	"Kuanyama; Kwanyama"},
		{"kum",	null,	null,	"Kumyk"},
		{"kur",	null,	"ku",	"Kurdish"},
		{"kut",	null,	null,	"Kutenai"},
		{"lad",	null,	null,	"Ladino"},
		{"lah",	null,	null,	"Lahnda"},
		{"lam",	null,	null,	"Lamba"},
		{"lao",	null,	"lo",	"Lao"},
		{"lat",	null,	"la",	"Latin"},
		{"lav",	null,	"lv",	"Latvian"},
		{"lez",	null,	null,	"Lezghian"},
		{"lim",	null,	"li",	"Limburgan; Limburger; Limburgish"},
		{"lin",	null,	"ln",	"Lingala"},
		{"lit",	null,	"lt",	"Lithuanian"},
		{"lol",	null,	null,	"Mongo"},
		{"loz",	null,	null,	"Lozi"},
		{"ltz",	null,	"lb",	"Luxembourgish; Letzeburgesch"},
		{"lua",	null,	null,	"Luba-Lulua"},
		{"lub",	null,	"lu",	"Luba-Katanga"},
		{"lug",	null,	"lg",	"Ganda"},
		{"lui",	null,	null,	"Luiseno"},
		{"lun",	null,	null,	"Lunda"},
		{"luo",	null,	null,	"Luo (Kenya and Tanzania)"},
		{"lus",	null,	null,	"Lushai"},
		{"mac",	"mkd",	"mk",	"Macedonian"},
		{"mad",	null,	null,	"Madurese"},
		{"mag",	null,	null,	"Magahi"},
		{"mah",	null,	"mh",	"Marshallese"},
		{"mai",	null,	null,	"Maithili"},
		{"mak",	null,	null,	"Makasar"},
		{"mal",	null,	"ml",	"Malayalam"},
		{"man",	null,	null,	"Mandingo"},
		{"mao",	"mri",	"mi",	"Maori"},
		{"map",	null,	null,	"Austronesian languages"},
		{"mar",	null,	"mr",	"Marathi"},
		{"mas",	null,	null,	"Masai"},
		{"may",	"msa",	"ms",	"Malay"},
		{"mdf",	null,	null,	"Moksha"},
		{"mdr",	null,	null,	"Mandar"},
		{"men",	null,	null,	"Mende"},
		{"mga",	null,	null,	"Irish, Middle (900-1200)"},
		{"mic",	null,	null,	"Mi'kmaq; Micmac"},
		{"min",	null,	null,	"Minangkabau"},
		{"mis",	null,	null,	"Uncoded languages"},
		{"mkh",	null,	null,	"Mon-Khmer languages"},
		{"mlg",	null,	"mg",	"Malagasy"},
		{"mlt",	null,	"mt",	"Maltese"},
		{"mnc",	null,	null,	"Manchu"},
		{"mni",	null,	null,	"Manipuri"},
		{"mno",	null,	null,	"Manobo languages"},
		{"moh",	null,	null,	"Mohawk"},
		{"mon",	null,	"mn",	"Mongolian"},
		{"mos",	null,	null,	"Mossi"},
		{"mul",	null,	null,	"Multiple languages"},
		{"mun",	null,	null,	"Munda languages"},
		{"mus",	null,	null,	"Creek"},
		{"mwl",	null,	null,	"Mirandese"},
		{"mwr",	null,	null,	"Marwari"},
		{"myn",	null,	null,	"Mayan languages"},
		{"myv",	null,	null,	"Erzya"},
		{"nah",	null,	null,	"Nahuatl languages"},
		{"nai",	null,	null,	"North American Indian languages"},
		{"nap",	null,	null,	"Neapolitan"},
		{"nau",	null,	"na",	"Nauru"},
		{"nav",	null,	"nv",	"Navajo; Navaho"},
		{"nbl",	null,	"nr",	"Ndebele, South; South Ndebele"},
		{"nde",	null,	"nd",	"Ndebele, North; North Ndebele"},
		{"ndo",	null,	"ng",	"Ndonga"},
		{"nds",	null,	null,	"Low German; Low Saxon; German, Low; Saxon, Low"},
		{"nep",	null,	"ne",	"Nepali"},
		{"new",	null,	null,	"Nepal Bhasa; Newari"},
		{"nia",	null,	null,	"Nias"},
		{"nic",	null,	null,	"Niger-Kordofanian languages"},
		{"niu",	null,	null,	"Niuean"},
		{"nno",	null,	"nn",	"Norwegian Nynorsk; Nynorsk, Norwegian"},
		{"nob",	null,	"nb",	"Bokmål, Norwegian; Norwegian Bokmål"},
		{"nog",	null,	null,	"Nogai"},
		{"non",	null,	null,	"Norse, Old"},
		{"nor",	null,	"no",	"Norwegian"},
		{"nqo",	null,	null,	"N'Ko"},
		{"nso",	null,	null,	"Pedi; Sepedi; Northern Sotho"},
		{"nub",	null,	null,	"Nubian languages"},
		{"nwc",	null,	null,	"Classical Newari; Old Newari; Classical Nepal Bhasa"},
		{"nya",	null,	"ny",	"Chichewa; Chewa; Nyanja"},
		{"nym",	null,	null,	"Nyamwezi"},
		{"nyn",	null,	null,	"Nyankole"},
		{"nyo",	null,	null,	"Nyoro"},
		{"nzi",	null,	null,	"Nzima"},
		{"oci",	null,	"oc",	"Occitan (post 1500)"},
		{"oji",	null,	"oj",	"Ojibwa"},
		{"ori",	null,	"or",	"Oriya"},
		{"orm",	null,	"om",	"Oromo"},
		{"osa",	null,	null,	"Osage"},
		{"oss",	null,	"os",	"Ossetian; Ossetic"},
		{"ota",	null,	null,	"Turkish, Ottoman (1500-1928)"},
		{"oto",	null,	null,	"Otomian languages"},
		{"paa",	null,	null,	"Papuan languages"},
		{"pag",	null,	null,	"Pangasinan"},
		{"pal",	null,	null,	"Pahlavi"},
		{"pam",	null,	null,	"Pampanga; Kapampangan"},
		{"pan",	null,	"pa",	"Panjabi; Punjabi"},
		{"pap",	null,	null,	"Papiamento"},
		{"pau",	null,	null,	"Palauan"},
		{"peo",	null,	null,	"Persian, Old (ca.600-400 B.C.)"},
		{"phi",	null,	null,	"Philippine languages"},
		{"phn",	null,	null,	"Phoenician"},
		{"pli",	null,	"pi",	"Pali"},
		{"pol",	null,	"pl",	"Polish"},
		{"pon",	null,	null,	"Pohnpeian"},
		{"por",	null,	"pt",	"Portuguese"},
		{"pra",	null,	null,	"Prakrit languages"},
		{"pro",	null,	null,	"Provençal, Old (to 1500);Occitan, Old (to 1500)"},
		{"pus",	null,	"ps",	"Pushto; Pashto"},
		//{"qaa-qtz",	null,	null,	"Reserved for local use"},
		{"que",	null,	"qu",	"Quechua"},
		{"raj",	null,	null,	"Rajasthani"},
		{"rap",	null,	null,	"Rapanui"},
		{"rar",	null,	null,	"Rarotongan; Cook Islands Maori"},
		{"roa",	null,	null,	"Romance languages"},
		{"roh",	null,	"rm",	"Romansh"},
		{"rom",	null,	null,	"Romany"},
		{"rum",	"ron",	"ro",	"Romanian; Moldavian; Moldovan"},
		{"run",	null,	"rn",	"Rundi"},
		{"rup",	null,	null,	"Aromanian; Arumanian; Macedo-Romanian"},
		{"rus",	null,	"ru",	"Russian"},
		{"sad",	null,	null,	"Sandawe"},
		{"sag",	null,	"sg",	"Sango"},
		{"sah",	null,	null,	"Yakut"},
		{"sai",	null,	null,	"South American Indian languages"},
		{"sal",	null,	null,	"Salishan languages"},
		{"sam",	null,	null,	"Samaritan Aramaic"},
		{"san",	null,	"sa",	"Sanskrit"},
		{"sas",	null,	null,	"Sasak"},
		{"sat",	null,	null,	"Santali"},
		{"scn",	null,	null,	"Sicilian"},
		{"sco",	null,	null,	"Scots"},
		{"sel",	null,	null,	"Selkup"},
		{"sem",	null,	null,	"Semitic languages"},
		{"sga",	null,	null,	"Irish, Old (to 900)"},
		{"sgn",	null,	null,	"Sign Languages"},
		{"shn",	null,	null,	"Shan"},
		{"sid",	null,	null,	"Sidamo"},
		{"sin",	null,	"si",	"Sinhala; Sinhalese"},
		{"sio",	null,	null,	"Siouan languages"},
		{"sit",	null,	null,	"Sino-Tibetan languages"},
		{"sla",	null,	null,	"Slavic languages"},
		{"slo",	"slk",	"sk",	"Slovak"},
		{"slv",	null,	"sl",	"Slovenian"},
		{"sma",	null,	null,	"Southern Sami"},
		{"sme",	null,	"se",	"Northern Sami"},
		{"smi",	null,	null,	"Sami languages"},
		{"smj",	null,	null,	"Lule Sami"},
		{"smn",	null,	null,	"Inari Sami"},
		{"smo",	null,	"sm",	"Samoan"},
		{"sms",	null,	null,	"Skolt Sami"},
		{"sna",	null,	"sn",	"Shona"},
		{"snd",	null,	"sd",	"Sindhi"},
		{"snk",	null,	null,	"Soninke"},
		{"sog",	null,	null,	"Sogdian"},
		{"som",	null,	"so",	"Somali"},
		{"son",	null,	null,	"Songhai languages"},
		{"sot",	null,	"st",	"Sotho, Southern"},
		{"spa",	null,	"es",	"Spanish; Castilian"},
		{"srd",	null,	"sc",	"Sardinian"},
		{"srn",	null,	null,	"Sranan Tongo"},
		{"srp",	null,	"sr",	"Serbian"},
		{"srr",	null,	null,	"Serer"},
		{"ssa",	null,	null,	"Nilo-Saharan languages"},
		{"ssw",	null,	"ss",	"Swati"},
		{"suk",	null,	null,	"Sukuma"},
		{"sun",	null,	"su",	"Sundanese"},
		{"sus",	null,	null,	"Susu"},
		{"sux",	null,	null,	"Sumerian"},
		{"swa",	null,	"sw",	"Swahili"},
		{"swe",	null,	"sv",	"Swedish"},
		{"syc",	null,	null,	"Classical Syriac"},
		{"syr",	null,	null,	"Syriac"},
		{"tah",	null,	"ty",	"Tahitian"},
		{"tai",	null,	null,	"Tai languages"},
		{"tam",	null,	"ta",	"Tamil"},
		{"tat",	null,	"tt",	"Tatar"},
		{"tel",	null,	"te",	"Telugu"},
		{"tem",	null,	null,	"Timne"},
		{"ter",	null,	null,	"Tereno"},
		{"tet",	null,	null,	"Tetum"},
		{"tgk",	null,	"tg",	"Tajik"},
		{"tgl",	null,	"tl",	"Tagalog"},
		{"tha",	null,	"th",	"Thai"},
		{"tig",	null,	null,	"Tigre"},
		{"tir",	null,	"ti",	"Tigrinya"},
		{"tiv",	null,	null,	"Tiv"},
		{"tkl",	null,	null,	"Tokelau"},
		{"tlh",	null,	null,	"Klingon; tlhIngan-Hol"},
		{"tli",	null,	null,	"Tlingit"},
		{"tmh",	null,	null,	"Tamashek"},
		{"tog",	null,	null,	"Tonga (Nyasa)"},
		{"ton",	null,	"to",	"Tonga (Tonga Islands)"},
		{"tpi",	null,	null,	"Tok Pisin"},
		{"tsi",	null,	null,	"Tsimshian"},
		{"tsn",	null,	"tn",	"Tswana"},
		{"tso",	null,	"ts",	"Tsonga"},
		{"tuk",	null,	"tk",	"Turkmen"},
		{"tum",	null,	null,	"Tumbuka"},
		{"tup",	null,	null,	"Tupi languages"},
		{"tur",	null,	"tr",	"Turkish"},
		{"tut",	null,	null,	"Altaic languages"},
		{"tvl",	null,	null,	"Tuvalu"},
		{"twi",	null,	"tw",	"Twi"},
		{"tyv",	null,	null,	"Tuvinian"},
		{"udm",	null,	null,	"Udmurt"},
		{"uga",	null,	null,	"Ugaritic"},
		{"uig",	null,	"ug",	"Uighur; Uyghur"},
		{"ukr",	null,	"uk",	"Ukrainian"},
		{"umb",	null,	null,	"Umbundu"},
		{"und",	null,	null,	"Undetermined"},
		{"urd",	null,	"ur",	"Urdu"},
		{"uzb",	null,	"uz",	"Uzbek"},
		{"vai",	null,	null,	"Vai"},
		{"ven",	null,	"ve",	"Venda"},
		{"vie",	null,	"vi",	"Vietnamese"},
		{"vol",	null,	"vo",	"Volapük"},
		{"vot",	null,	null,	"Votic"},
		{"wak",	null,	null,	"Wakashan languages"},
		{"wal",	null,	null,	"Wolaitta; Wolaytta"},
		{"war",	null,	null,	"Waray"},
		{"was",	null,	null,	"Washo"},
		{"wen",	null,	null,	"Sorbian languages"},
		{"wln",	null,	"wa",	"Walloon"},
		{"wol",	null,	"wo",	"Wolof"},
		{"xal",	null,	null,	"Kalmyk; Oirat"},
		{"xho",	null,	"xh",	"Xhosa"},
		{"yao",	null,	null,	"Yao"},
		{"yap",	null,	null,	"Yapese"},
		{"yid",	null,	"yi",	"Yiddish"},
		{"yor",	null,	"yo",	"Yoruba"},
		{"ypk",	null,	null,	"Yupik languages"},
		{"zap",	null,	null,	"Zapotec"},
		{"zbl",	null,	null,	"Blissymbols; Blissymbolics; Bliss"},
		{"zen",	null,	null,	"Zenaga"},
		{"zha",	null,	"za",	"Zhuang; Chuang"},
		{"znd",	null,	null,	"Zande languages"},
		{"zul",	null,	"zu",	"Zulu"},
		{"zun",	null,	null,	"Zuni"},
		{"zxx",	null,	null,	"No linguistic content; Not applicable"},
		{"zza",	null,	null,	"Zaza; Dimili; Dimli; Kirdki; Kirmanjki; Zazaki"}
	};
	
}
